package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ExternalDatasetDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.ExternalDatasetDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.ExternalDatasetDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.ExternalDatasetData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.ExternalDatasetDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.validation.Validator;
import gr.cite.tools.validation.ValidatorFactory;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class ExternalDatasetFieldDataHelperService extends BaseFieldDataHelperService<ExternalDatasetData, ExternalDatasetDataPersist, ExternalDatasetDataEntity, ExternalDatasetDataImportExport> {
	
	private final BuilderFactory builderFactory;
	private final ValidatorFactory validatorFactory;

	public ExternalDatasetFieldDataHelperService(BuilderFactory builderFactory, ValidatorFactory validatorFactory) {
		this.builderFactory = builderFactory;
		this.validatorFactory = validatorFactory;
	}

	@Override
	public ExternalDatasetDataEntity newDataInstanceInternal() {
		return new ExternalDatasetDataEntity();
	}

	@Override
	public ExternalDatasetData newModelInstanceInternal() {
		return new ExternalDatasetData();
	}

	@Override
	public ExternalDatasetDataPersist newPersistModelInstanceInternal() {
		return new ExternalDatasetDataPersist();
	}

	@Override
	protected ExternalDatasetDataImportExport newImportExportInstanceInternal() {
		return new ExternalDatasetDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return ExternalDatasetDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return ExternalDatasetData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return ExternalDatasetDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return ExternalDatasetDataImportExport.class;
	}

	@Override
	public Validator getPersistModelValidator() {
		return this.validatorFactory.validator(ExternalDatasetDataPersist.ExternalDatasetDataPersistValidator.class);
	}
	@Override
	public List<ExternalDatasetData> buildInternal(FieldSet fieldSet, List<ExternalDatasetDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(ExternalDatasetDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected ExternalDatasetDataEntity applyPersistInternal(ExternalDatasetDataPersist persist, ExternalDatasetDataEntity data) {
		data.setMultipleSelect(persist.getMultipleSelect());
		data.setType(persist.getType());
		return data;
	}

	@Override
	protected ExternalDatasetDataPersist importExportMapDataToPersistInternal(ExternalDatasetDataImportExport data, ExternalDatasetDataPersist persist) {
		persist.setMultipleSelect(data.getMultipleSelect());
		persist.setType(data.getType());
		return persist;
	}

	@Override
	protected ExternalDatasetDataImportExport dataToImportExportXmlInternal(ExternalDatasetDataEntity data, ExternalDatasetDataImportExport xml) {
		xml.setMultipleSelect(data.getMultipleSelect());
		xml.setType(data.getType());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(ExternalDatasetDataEntity data) {
		return data.getMultipleSelect();
	}
}
