package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ReferenceTypeDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.ReferenceTypeDataImportExport;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.ReferenceTypeDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.ReferenceTypeData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.ReferenceTypeDataPersist;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.validation.Validator;
import gr.cite.tools.validation.ValidatorFactory;
import org.springframework.stereotype.Component;

import java.util.EnumSet;
import java.util.List;

@Component
public class ReferenceTypeFieldDataHelperService extends BaseFieldDataHelperService<ReferenceTypeData, ReferenceTypeDataPersist, ReferenceTypeDataEntity, ReferenceTypeDataImportExport> {
	
	private final BuilderFactory builderFactory;

	private final ValidatorFactory validatorFactory;
	public ReferenceTypeFieldDataHelperService(BuilderFactory builderFactory, ValidatorFactory validatorFactory) {
		this.builderFactory = builderFactory;
		this.validatorFactory = validatorFactory;
	}

	@Override
	public ReferenceTypeDataEntity newDataInstanceInternal() {
		return new ReferenceTypeDataEntity();
	}

	@Override
	public ReferenceTypeData newModelInstanceInternal() {
		return new ReferenceTypeData();
	}

	@Override
	public ReferenceTypeDataPersist newPersistModelInstanceInternal() {
		return new ReferenceTypeDataPersist();
	}

	@Override
	protected ReferenceTypeDataImportExport newImportExportInstanceInternal() {
		return new ReferenceTypeDataImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return ReferenceTypeDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return ReferenceTypeData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return ReferenceTypeDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return ReferenceTypeDataImportExport.class;
	}
	
	@Override
	public Validator getPersistModelValidator() {
		return this.validatorFactory.validator(ReferenceTypeDataPersist.ReferenceTypeDataPersistValidator.class);
	}

	@Override
	public List<ReferenceTypeData> buildInternal(FieldSet fieldSet, List<ReferenceTypeDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(ReferenceTypeDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected ReferenceTypeDataEntity applyPersistInternal(ReferenceTypeDataPersist persist, ReferenceTypeDataEntity data) {
		data.setMultipleSelect(persist.getMultipleSelect());
		data.setReferenceTypeId(persist.getReferenceTypeId());
		return data;
	}

	@Override
	protected ReferenceTypeDataPersist importExportMapDataToPersistInternal(ReferenceTypeDataImportExport data, ReferenceTypeDataPersist persist){
		persist.setMultipleSelect(data.getMultipleSelect());
		persist.setReferenceTypeId(data.getReferenceTypeId());
		return persist;
	}

	@Override
	protected ReferenceTypeDataImportExport dataToImportExportXmlInternal(ReferenceTypeDataEntity data, ReferenceTypeDataImportExport xml) {
		xml.setMultipleSelect(data.getMultipleSelect());
		xml.setReferenceTypeId(data.getReferenceTypeId());
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(ReferenceTypeDataEntity data) {
		return data.getMultipleSelect();
	}
}
