package eu.eudat.controllers;

import eu.eudat.authorization.Permission;
import eu.eudat.logic.managers.ValidationManager;
import eu.eudat.commons.exceptions.HugeResultSetException;
import eu.eudat.models.old.helpers.responses.ResponseItem;
import eu.eudat.types.ApiMessageCode;
import gr.cite.commons.web.authz.service.AuthorizationService;
import gr.cite.tools.exception.MyNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@CrossOrigin
@RequestMapping(value = {"/api"})
public class Validation  {

	private ValidationManager validationManager;
	private final AuthorizationService authorizationService;

	@Autowired
	public Validation(ValidationManager validationManager, AuthorizationService authorizationService) {
		this.validationManager = validationManager;
		this.authorizationService = authorizationService;
	}

	@RequestMapping(method = RequestMethod.GET, value = {"/external/validation"}, produces = "application/json")
	public @ResponseBody
	ResponseEntity<ResponseItem<Boolean>> validate(
			@RequestParam(value = "query", required = false) String query, @RequestParam(value = "type", required = false) String type
	) throws HugeResultSetException, MyNotFoundException {
		this.authorizationService.authorizeForce(Permission.AuthenticatedRole);

		Boolean isValid = this.validationManager.validateIdentifier(query, type);
		return ResponseEntity.status(HttpStatus.OK).body(new ResponseItem<Boolean>().payload(isValid).status(ApiMessageCode.NO_MESSAGE));
	}
}

