import { Component, OnInit } from '@angular/core';
import { BaseComponent } from '@common/base/base.component';
// import { BreadcrumbItem } from '@app/ui/misc/breadcrumb/definition/breadcrumb-item';
import { Location } from '@angular/common';
import { UntypedFormControl } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Params, Router } from '@angular/router';
import { DescriptionStatus } from '@app/core/common/enum/description-status';
import { DmpAccessType } from '@app/core/common/enum/dmp-access-type';
import { DmpStatus } from '@app/core/common/enum/dmp-status';
import { DmpUserRole } from '@app/core/common/enum/dmp-user-role';
import { DescriptionTemplate } from '@app/core/model/description-template/description-template';
import { Description, DescriptionStatusPersist } from '@app/core/model/description/description';
import { Dmp, DmpUser, DmpUserRemovePersist } from '@app/core/model/dmp/dmp';
import { DmpReference } from '@app/core/model/dmp/dmp-reference';
import { Reference } from '@app/core/model/reference/reference';
import { AuthService } from '@app/core/services/auth/auth.service';
import { ConfigurationService } from '@app/core/services/configuration/configuration.service';
import { DescriptionService } from '@app/core/services/description/description.service';
import { DmpService } from '@app/core/services/dmp/dmp.service';
import { FileTransformerService } from '@app/core/services/file-transformer/file-transformer.service';
import { MatomoService } from '@app/core/services/matomo/matomo-service';
import { SnackBarNotificationLevel, UiNotificationService } from '@app/core/services/notification/ui-notification-service';
import { ReferenceTypeService } from '@app/core/services/reference-type/reference-type.service';
import { ReferenceService } from '@app/core/services/reference/reference.service';
import { EnumUtils } from '@app/core/services/utilities/enum-utils.service';
import { FileUtils } from '@app/core/services/utilities/file-utils.service';
import { ConfirmationDialogComponent } from '@common/modules/confirmation-dialog/confirmation-dialog.component';
import { Guid } from '@common/types/guid';
import { TranslateService } from '@ngx-translate/core';
import * as FileSaver from 'file-saver';
import { filter, takeUntil } from 'rxjs/operators';
import { nameof } from 'ts-simple-nameof';
import { DescriptionCopyDialogComponent } from '../description-copy-dialog/description-copy-dialog.component';


@Component({
	selector: 'app-description-overview',
	templateUrl: './description-overview.component.html',
	styleUrls: ['./description-overview.component.scss']
})
export class DescriptionOverviewComponent extends BaseComponent implements OnInit {

	description: Description;
	researchers: DmpReference[] = [];
	isNew = true;
	isFinalized = false;
	isPublicView = true;
	hasPublishButton: boolean = true;
	// breadCrumbs: Observable<BreadcrumbItem[]> = observableOf();
	isUserOwner: boolean;
	expand = false;
	lockStatus: Boolean;
	descriptionStatusEnum = DescriptionStatus;
	dmpAccessTypeEnum = DmpAccessType;
	dmpStatusEnum = DmpStatus;
	dmpUserRoleEnum = DmpUserRole;

	constructor(
		private route: ActivatedRoute,
		private router: Router,
		private descriptionService: DescriptionService,
		private authentication: AuthService,
		private dialog: MatDialog,
		private language: TranslateService,
		private uiNotificationService: UiNotificationService,
		private configurationService: ConfigurationService,
		private dmpService: DmpService,
		public referenceService: ReferenceService,
		private location: Location,
		public enumUtils: EnumUtils,
		private matomoService: MatomoService,
		private fileUtils: FileUtils,
		public fileTransformerService: FileTransformerService,
		private referenceTypeService: ReferenceTypeService
	) {
		super();
	}

	ngOnInit() {
		this.matomoService.trackPageView('Description Overview');
		// Gets description data using parameter id
		this.route.params
			.pipe(takeUntil(this._destroyed))
			.subscribe((params: Params) => {
				const itemId = params['id'];
				const publicId = params['publicId'];
				if (itemId != null) {
					this.isNew = false;
					this.isPublicView = false;
					this.descriptionService.getSingle(itemId, this.lookupFields())
						.pipe(takeUntil(this._destroyed))
						.subscribe(data => {
							this.description = data;
							this.researchers = this.referenceService.getReferencesForTypes(this.description?.dmp?.dmpReferences, [this.referenceTypeService.getResearcherReferenceType()]);
							// this.users = this.description.dmp.users;
							this.checkLockStatus(this.description.id);
							this.setIsUserOwner();
							// const breadCrumbs = [];
							// breadCrumbs.push({ parentComponentName: null, label: this.language.instant('NAV-BAR.MY-DESCRIPTION-DESCRIPTIONS'), url: "/descriptions" });
							// breadCrumbs.push({ parentComponentName: 'DescriptionListingComponent', label: this.description.label, url: '/descriptions/overview/' + this.description.id });
							// this.breadCrumbs = observableOf(breadCrumbs);
						}, (error: any) => {
							if (error.status === 404) {
								return this.onFetchingDeletedCallbackError('/descriptions/');
							}
							if (error.status === 403) {
								return this.onFetchingForbiddenCallbackError('/descriptions/');
							}
						});
				}
				else if (publicId != null) {
					this.isNew = false;
					this.isFinalized = true;
					this.isPublicView = true;
					this.descriptionService.getPublicSingle(publicId, this.lookupFields())
						.pipe(takeUntil(this._destroyed))
						.subscribe(data => {
							this.description = data;
							this.researchers = this.referenceService.getReferencesForTypes(this.description?.dmp?.dmpReferences, [this.referenceTypeService.getResearcherReferenceType()]);
							// this.users = this.description.dmp.users;
							// const breadCrumbs = [];
							// breadCrumbs.push({ parentComponentName: null, label: this.language.instant('NAV-BAR.PUBLIC DESCRIPTIONS'), url: "/explore" });
							// breadCrumbs.push({ parentComponentName: 'DescriptionListingComponent', label: this.description.label, url: '/descriptions/overview/public/' + this.description.id });
							// this.breadCrumbs = observableOf(breadCrumbs);
						}, (error: any) => {
							if (error.status === 404) {
								return this.onFetchingDeletedCallbackError('/explore-descriptions');
							}
							if (error.status === 403) {
								return this.onFetchingForbiddenCallbackError('/explore-descriptions');
							}
						});
				}
			});
	}

	checkLockStatus(id: Guid) {
		// TODO: add this
		// this.lockService.checkLockStatus(id).pipe(takeUntil(this._destroyed))
		// 	.subscribe(lockStatus => {
		// 		this.lockStatus = lockStatus
		// 		if (lockStatus) {
		// 			this.dialog.open(PopupNotificationDialogComponent, {
		// 				data: {
		// 					title: this.language.instant('DESCRIPTION-OVERVIEW.LOCKED-DIALOG.TITLE'),
		// 					message: this.language.instant('DESCRIPTION-OVERVIEW.LOCKED-DIALOG.MESSAGE')
		// 				}, maxWidth: '30em'
		// 			});
		// 		}
		// 	});
	}

	onFetchingDeletedCallbackError(redirectRoot: string) {
		this.uiNotificationService.snackBarNotification(this.language.instant('DESCRIPTION-OVERVIEW.ERROR.DELETED-DESCRIPTION'), SnackBarNotificationLevel.Error);
		this.router.navigate([redirectRoot]);
	}

	onFetchingForbiddenCallbackError(redirectRoot: string) {
		this.uiNotificationService.snackBarNotification(this.language.instant('DESCRIPTION-OVERVIEW.ERROR.FORBIDEN-DESCRIPTION'), SnackBarNotificationLevel.Error);
		this.router.navigate([redirectRoot]);
	}

	goBack(): void {
		this.location.back();
	}

	reloadPage(): void {
		const path = this.location.path();
		this.router.navigateByUrl('/reload', { skipLocationChange: true }).then(() => this.router.navigate([path]));
	}

	setIsUserOwner() {
		if (this.description) {
			const principalId: Guid = this.authentication.userId();
			if (principalId) this.isUserOwner = !!this.description.dmp.dmpUsers.find(x => (x.role === DmpUserRole.Owner) && (principalId === x.id));
		}
	}

	isUserAuthor(userId: Guid): boolean {
		if (this.isAuthenticated()) {
			const principalId: Guid = this.authentication.userId();
			return userId === principalId;
		} else return false;
	}

	isUserDescriptionRelated(): boolean {
		const principalId: Guid = this.authentication.userId();
		return this.description.dmp.dmpUsers.some(x => (x.user.id === principalId));
	}


	openShareDialog(rowId: any, rowName: any) {
		// TODO: add dialog
		// const dialogRef = this.dialog.open(DmpInvitationDialogComponent, {
		// 	autoFocus: false,
		// 	restoreFocus: false,
		// 	data: {
		// 		dmpId: rowId,
		// 		dmpName: rowName
		// 	}
		// });
	}

	public isAuthenticated(): boolean {
		return this.authentication.currentAccountIsAuthenticated();
	}

	isDraftDescription(description: Description) {
		return description.status == DescriptionStatus.Draft;
	}

	editClicked(description: Description) {
		if (description.status === DescriptionStatus.Finalized && description.dmp.accessType === DmpAccessType.Public) {
			this.router.navigate(['/descriptions/publicEdit/', description.id]);
		} else {
			this.router.navigate(['/descriptions/edit/', description.id]);
		}
	}

	deleteClicked() {
		const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
			maxWidth: '300px',
			data: {
				message: this.language.instant('GENERAL.CONFIRMATION-DIALOG.DELETE-ITEM'),
				confirmButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.DELETE'),
				cancelButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.CANCEL'),
				isDeleteConfirmation: true
			}
		});
		dialogRef.afterClosed().pipe(takeUntil(this._destroyed)).subscribe(result => {
			if (result) {
				this.descriptionService.delete(this.description.id)
					.pipe(takeUntil(this._destroyed))
					.subscribe(
						complete => {
							this.onDeleteCallbackSuccess();
						},
						error => this.onDeleteCallbackError(error)
					);
			}
		});
	}

	dmpClicked(dmp: Dmp) {
		if (this.isPublicView) {
			this.router.navigate(['/explore-plans/overview/public/' + dmp.id]);
		} else {
			this.router.navigate(['/plans/overview/' + dmp.id]);
		}
	}

	private onDeleteCallbackSuccess(): void {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-DELETE'), SnackBarNotificationLevel.Success);
		this.router.navigate(['/descriptions']);
	}

	private onDeleteCallbackError(error) {
		this.uiNotificationService.snackBarNotification(error.error.message ? error.error.message : this.language.instant('GENERAL.SNACK-BAR.UNSUCCESSFUL-DELETE'), SnackBarNotificationLevel.Error);
	}

	private onUpdateCallbackSuccess(): void {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE'), SnackBarNotificationLevel.Success);
		this.reloadPage();
	}

	private onUpdateCallbackError(error) {
		this.uiNotificationService.snackBarNotification(error.error.message ? this.tryTranslate(error.error.message) : this.language.instant('DESCRIPTION-UPLOAD.SNACK-BAR.UNSUCCESSFUL'), SnackBarNotificationLevel.Error);
	}

	private tryTranslate(errorMessage: string): string {
		return errorMessage.replace('Field value of', this.language.instant('Field value of'))
			.replace('must be filled', this.language.instant('must be filled'));
	}

	public getOrcidPath(): string {
		return this.configurationService.orcidPath;
	}

	isOrcid(reference: Reference) {
		return reference.source === 'orcid';
	}

	getOrcidPathForResearcher(reference: string): string {
		const path = this.getOrcidPath();
		return path + reference;
	}

	// downloadPDF(id: string) {
	// 	this.descriptionService.downloadPDF(id)
	// 		.pipe(takeUntil(this._destroyed))
	// 		.subscribe(response => {
	// 			const blob = new Blob([response.body], { type: 'application/pdf' });
	// 			const filename = this.fileUtils.getFilenameFromContentDispositionHeader(response.headers.get('Content-Disposition'));

	// 			FileSaver.saveAs(blob, filename);
	// 			this.matomoService.trackDownload('descriptions', "pdf", id);
	// 		});
	// }

	download(id: string, format: string) {
		this.descriptionService.download(id, format)
			.pipe(takeUntil(this._destroyed))
			.subscribe(response => {
				const blob = new Blob([response.body], { type: 'application/octet-stream' });
				const filename = this.fileUtils.getFilenameFromContentDispositionHeader(response.headers.get('Content-Disposition'));

				FileSaver.saveAs(blob, filename);
				this.matomoService.trackDownload('descriptions', format, id);
			});
	}

	downloadXml(id: string) {
		// TODO: add download
		// this.descriptionService.downloadXML(id)
		// 	.pipe(takeUntil(this._destroyed))
		// 	.subscribe(response => {
		// 		const blob = new Blob([response.body], { type: 'application/xml' });
		// 		const filename = this.fileUtils.getFilenameFromContentDispositionHeader(response.headers.get('Content-Disposition'));

		// 		FileSaver.saveAs(blob, filename);
		// 		this.matomoService.trackDownload('descriptions', "xml", id);
		// 	});
	}

	openCopyToDmpDialog() {
		const formControl = new UntypedFormControl();
		const dialogRef = this.dialog.open(DescriptionCopyDialogComponent, {
			width: '500px',
			restoreFocus: false,
			data: {
				formControl: formControl,
				descriptionId: this.description.id,
				descriptionTemplate: this.description.descriptionTemplate,
				descriptionProfileExist: false,
				confirmButton: this.language.instant('DESCRIPTION-OVERVIEW.COPY-DIALOG.COPY'),
				cancelButton: this.language.instant('DESCRIPTION-OVERVIEW.COPY-DIALOG.CANCEL')
			}
		});

		dialogRef.afterClosed().pipe(takeUntil(this._destroyed))
			.subscribe(newDmpId => {
				if (newDmpId) {
					this.router.navigate(['/descriptions/copy/' + this.description.id], { queryParams: { newDmpId: newDmpId } });
					// let url = this.router.createUrlTree(['/descriptions/copy/', result.descriptionId, { newDmpId: newDmpId }])
					// window.open(url.toString(), '_blank')
				}
			});
	}

	removeUserFromDmp(dmpUser: DmpUser) {
		const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
			data: {
				message: this.language.instant('GENERAL.CONFIRMATION-DIALOG.DELETE-USER'),
				confirmButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.REMOVE'),
				cancelButton: this.language.instant('GENERAL.CONFIRMATION-DIALOG.ACTIONS.CANCEL'),
				isDeleteConfirmation: false
			}
		});
		dialogRef.afterClosed().subscribe(result => {
			if (result) {
				const dmpUserRemovePersist: DmpUserRemovePersist = {
					id: dmpUser.id,
					dmpId: this.description.dmp.id,
					role: dmpUser.role
				};
				this.dmpService.removeUser(dmpUserRemovePersist).pipe(takeUntil(this._destroyed))
					.subscribe(data => {
						this.reloadPage();
					}, (error: any) => {
						//TODO: Error handling
					});

			}
		});
	}

	finalize(description: Description) {

		this.dialog.open(ConfirmationDialogComponent, {
			data: {
				message: this.language.instant('DESCRIPTION-OVERVIEW.FINALISE-POPUP.MESSAGE'),
				confirmButton: this.language.instant('DESCRIPTION-OVERVIEW.FINALISE-POPUP.CONFIRM'),
				cancelButton: this.language.instant('DESCRIPTION-OVERVIEW.FINALISE-POPUP.CANCEL'),
			},
			maxWidth: '30em'
		})
			.afterClosed()
			.pipe(
				filter(x => x),
				takeUntil(this._destroyed)
			)
			.subscribe(_ => {
				this.router.navigate(['descriptions', 'edit', description.id, 'finalize']);
			})
	}

	hasReversableStatus(description: Description): boolean {
		return description.dmp.status == DmpStatus.Draft && description.status == DescriptionStatus.Finalized
	}

	reverseFinalization(description: Description) {
		const dialogRef = this.dialog.open(ConfirmationDialogComponent, {
			restoreFocus: false,
			data: {
				message: this.language.instant('DESCRIPTION-OVERVIEW.UNDO-FINALIZATION-DIALOG.TITLE'),
				confirmButton: this.language.instant('DESCRIPTION-OVERVIEW.UNDO-FINALIZATION-DIALOG.CONFIRM'),
				cancelButton: this.language.instant('DESCRIPTION-OVERVIEW.UNDO-FINALIZATION-DIALOG.CANCEL'),
				isDeleteConfirmation: false
			}
		});
		dialogRef.afterClosed().pipe(takeUntil(this._destroyed)).subscribe(result => {
			if (result) {
				const dmpUserRemovePersist: DescriptionStatusPersist = {
					id: description.id,
					status: DescriptionStatus.Draft,
					hash: description.hash
				};
				this.descriptionService.persistStatus(dmpUserRemovePersist).pipe(takeUntil(this._destroyed))
					.subscribe(data => {
						this.reloadPage();
						this.onUpdateCallbackSuccess()
					}, (error: any) => {
						this.onUpdateCallbackError(error)
					});
			}
		});
	}

	private lookupFields(): string[] {
		return [
			nameof<Description>(x => x.id),
			nameof<Description>(x => x.label),
			nameof<Description>(x => x.status),
			nameof<Description>(x => x.updatedAt),
			[nameof<Description>(x => x.descriptionTemplate), nameof<DescriptionTemplate>(x => x.id)].join('.'),
			[nameof<Description>(x => x.descriptionTemplate), nameof<DescriptionTemplate>(x => x.label)].join('.'),
			[nameof<Description>(x => x.descriptionTemplate), nameof<DescriptionTemplate>(x => x.groupId)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.label)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.accessType)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.user.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.user.name)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpUsers), nameof<DmpUser>(x => x.role)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.id)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.label)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.type)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.source)].join('.'),
			[nameof<Description>(x => x.dmp), nameof<Dmp>(x => x.dmpReferences), nameof<DmpReference>(x => x.reference), nameof<Reference>(x => x.reference)].join('.'),
		]
	}

}
